/*-*-C-*-*/
/**
 *  Connection.java
 *  copyright (c) 1996, 1997 COBITE, Julian Haight and Robert Lippman
 */
import java.io.*;
import java.net.*;

public class Connection {
  private boolean connected;
  private Socket socket;
  public DataInputStream in;
  public DataOutputStream out;
  
  public Connection() {
    connected = false;
    in = null;
    out = null;
  }

  public void finalize() {
    if (connected) disconnect();
  }
 
  public boolean isConnected() {
    return connected;
  }

  public boolean connect(String hostname, int port) {
    InetAddress ineta;
 
    if (connected && !disconnect()) return false;

    try {
      ineta = java.net.InetAddress.getByName(hostname);
    } catch (UnknownHostException e) {
      System.err.println("Unknown host");
      return false;
    }
    
    try { 
      socket = new Socket(ineta, port);
    } catch (IOException e) {
      System.err.println("Error on connect");
      disconnect();
      return false;
    }
    
    try { in = new DataInputStream(socket.getInputStream());
    } catch (IOException e) {
      System.err.println("Error opening input stream");
      disconnect();
      return false;
    }
    
    try { out = new DataOutputStream(socket.getOutputStream());
    } catch (IOException e) {
      System.err.println("Error opening output stream");
      disconnect();
      return false;
    }
    
    connected = true;
    return true;
  }
  
  public boolean disconnect() {
    if (!connected) return true;

    try { socket.close();
    } catch (Exception e) {
      System.err.println(e);
      return false;
    }    
    
    connected = false;
    /*System.err.println("Disconnected");*/
    return true;
  }
}

