/*-*-C-*-*/

import java.awt.*;
import java.util.*;
import java.awt.event.*;

public class Filmer extends Frame implements WindowListener, ActionListener {
  FrameSet previews = new FrameSet(this);
  MapControls mapcontrols = new MapControls();
  RenderMan film = new RenderMan(mapcontrols, this);
  String filename = null;
  final static int NOTHING = -1;
  final static int PREVIEW = 0;
  final static int DORENDER = 1;
  final static int PREFILM = 2;
  int command = PREVIEW;
  Panel controls;
  Button buttons[];
  String button_labels[] = {"Open .par...", "Save as...", null,
			    "Start Render", "Stop Render", "Delete old Files",
			    null, "New Map", "Quit"};
  String button_commands[] = {"open", "save", null, "start", "stop", "delete",
			      null, "map", "quit"};
  Choice aspect_menu;
  
  public Filmer() {
    super();
    Label temp;
    int i;

    System.out.println("Filmer V2.0 (C) 1998 Julian Haight.  " + 
		       "All rights reserved.");

    controls = new Panel();
    controls.setForeground(Color.black);
    controls.setLayout(new GridLayout(button_labels.length + 2,1));
    buttons = new Button[button_labels.length];

    for (i=0; i < button_labels.length; i++) {
      if (button_labels[i] != null) {
	buttons[i] = new Button(button_labels[i]);
	buttons[i].addActionListener(this);
	buttons[i].setActionCommand(button_commands[i]);
	controls.add(buttons[i]);
      } else {
	controls.add(new Panel());
      }
    }

    temp = new Label("Aspect");
    temp.setForeground(Color.white);

    aspect_menu = new Choice();
    initAspect();

    controls.add(temp);
    controls.add(aspect_menu);

    mapcontrols.add("North", controls);

    FileManager.resetPreview();

    setLayout(new BorderLayout());
    setBackground(Color.black);
    setForeground(Color.white);
    add("West", mapcontrols);
    add("Center", previews);
    add("East", film);
    addWindowListener(this);

    pack();
    show();
  }

  void initAspect() {
    String aspects[] = FileManager.aspects();
    int i;

    for (i=0; i < aspects.length; i++) {
      aspect_menu.add(aspects[i]);
    }   
    aspect_menu.select(FileManager.lookup("<default_aspect>"));
  }
  
  boolean open() {
    String f = FileManager.getFile(this, "*.par", "Open .PAR file");
    stop();
    if (f != null) {
      return load(f);
    } else {
      return false;
    }
  }

  public boolean load(String f) {
    filename = f;
    stop();

    if (previews.loadPar(filename)) {
      remove(film);
      add("East", film = new RenderMan(mapcontrols, this));
      film.setFilm(previews, aspect_menu.getSelectedItem(), null);
      doLayout();
      System.gc();
      command = PREVIEW;
      wakeup();
      return true;
    } else {
      setMessage(".par load failed");
      film = null;
    }
    return false;
  }

  public void calc() {
    try {
      setSensitive(false);
      while(true) {
	if (command == PREVIEW && previews != null) {
	  previews.preView();
	  film.preView();
	  film.doLayout();
	  /*doLayout();*/
	  /*pack();*/
	}
	if (command == DORENDER && film != null) {
	  film.doIt();
	}
	synchronized(this) {
	  setMessage("V2.0 (C) 1998 Julian Haight - ready");
	  setSensitive(true);
	  wait();
	  setSensitive(false);
	}
      }
    } catch (java.lang.InterruptedException e) {
    }
  }

  void setSensitive(boolean on) {
    int i;
    for (i=0; i < button_labels.length; i++) {
      if (button_labels[i] != null &&
	  !button_commands[i].equals("quit")) {
	if (button_commands[i].equals("stop")) {
	  buttons[i].setEnabled(!on);
	} else {
	  buttons[i].setEnabled(on);
	}
      }
    }
    aspect_menu.setEnabled(on);
    mapcontrols.setSensitive(on);
  }

  public synchronized void wakeup() {
    notify();
  }

  public void map() {
    if (film != null) {
      film.newMap();
      command = PREVIEW;
      wakeup();
    }
  }

  void start() {
    if (previews != null) {
      film.setFilm(previews, aspect_menu.getSelectedItem(), null);
      command = DORENDER;
      wakeup();
    }
  }

  void stop() {
    setMessage("stopping...");

    if (previews != null) {
      previews.stop();
    }
    if (film != null) {
      film.stop();
    }
    setMessage("stopped");
  }

  void delete() {
    film.reset();
    setMessage("deleting temp files");
    FileManager.resetFilm();
    setMessage("ready");
  }

  void save() {
    String filename = FileManager.putFile(this, "*.par", "Save .PAR file");
    if (filename != null && previews.keyframes != null) {
      film.setFilm(previews, aspect_menu.getSelectedItem(), filename);
      command = DORENDER;
      wakeup();
    }
  }

  public void actionPerformed(ActionEvent e) {
    doCmd(e.getActionCommand());
  }

  void doCmd(String cmd) {
    if (cmd.equals("quit")) {
      quit();
    } else if (cmd.equals("open")) {
      open();
    } else if (cmd.equals("start")) {
      start();
    } else if (cmd.equals("save")) {
      save();
    } else if (cmd.equals("stop")) {
      stop();
    } else if (cmd.equals("map")) {
      map();
    } else if (cmd.equals("delete")) {
      delete();
    }
  }

  public void quit() {
    FileManager.resetPreview();
    Runtime.getRuntime().exit(0);
  }

  public void setMessage(String message) {
    StringBuffer text = new StringBuffer();
    text.append("Filmer");
    if (filename != null) {
      text.append("  (");
      text.append(filename);
      text.append(")");
    }
    if (message != null) {
      text.append(" - ");
      text.append(message);
    }
    setTitle(text.toString());
  }

  /* all this, just to trap the stupid little X */
  public void windowClosing(WindowEvent e) { quit(); }
  public void windowActivated(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}
  /* ugh */

  public static void main(String argv[]) {
    Thread t = Thread.currentThread();
    
    Filmer f = new Filmer();
    if (argv.length >= 1) {
      f.load(argv[0]);
    }
    t.setPriority(t.getPriority() - 1);
    f.calc();
  }
}

