/*-*-C-*-*/

import java.awt.*;
import java.util.*;
import java.io.*;

public class FrameSet extends ScrollPane {
  boolean previewed = false;
  public Vector keyframes;
  boolean timetodie;
  ParFrame calc;
  Panel display = new Panel();
  Filmer owner = null;
  
  /*final static int WIDTH=3;*/

  public FrameSet(Filmer o) {
    super();
    owner = o;
    getVAdjustable().setUnitIncrement(200);
    getHAdjustable().setUnitIncrement(200);
    add("Center", display);
  }

  public Dimension getPreferredSize() {
    return new Dimension(550, 500);
  }

  public synchronized Enumeration elements() {
    return keyframes.elements();
  }

  public void setBounds(int x, int y, int w, int h) {
    int parwidth, width, rows;

    if (keyframes != null && keyframes.size() > 0) {
      parwidth = ((ParFrame)keyframes.elementAt(0)).getPreferredSize().width;
    } else {
      parwidth = 259;
    }
    parwidth += 5;
    width = (w - getVScrollbarWidth()) / parwidth;
    if (width < 1) width = 1;

    if (keyframes != null) {
      rows = (int)(keyframes.size() / width);
      if ((float)rows < ((float)keyframes.size() / width)) rows++;
    } else {
      rows = 0;
    }

    /*System.out.println("FrameSet.setBounds width = "+width + 
		       " VSwidth=" + getVScrollbarWidth() +
		       " parwidth=" + parwidth +
		       " rows=" + rows);*/
    display.setLayout(new GridLayout(rows, width, 5, 5));

    doLayout();
    
    super.setBounds(x, y, w, h);
    repaint(100);
  }

  public void addElement(ParFrame frame) {
    keyframes.addElement(frame);
    display.add(frame);
    display.doLayout();
  }

  public void preView() {
    synchronized(this) {
      if (previewed == true || keyframes == null) {
	return;   /* return if we've already done it, or are doing it */
      } else {
	previewed = true;
      }
    }

    /*System.out.println("FrameSet.preview");*/
    timetodie = false;
    Enumeration e=keyframes.elements();
    while (e.hasMoreElements() && !timetodie) {
      calc = (ParFrame)e.nextElement();
      owner.setMessage("Calculating image for keyframe:" + calc.name());
      calc.preView();             /* takes a long time */
      doLayout();
      validate();
    }
    calc = null;
    if (timetodie) {
      owner.setMessage("Aborted calculating previews");
    } else {
      owner.setMessage("Done previewing images");
    }
  }

  public void stop() {
    timetodie = true;
    if (calc != null) {
      calc.stop();
    }
  }

  public synchronized boolean loadPar(String filename) {
    int method = -1;
    BufferedReader in;
    ParFrame current = new ParFrame(true);
    boolean firstframe = true;

    display.removeAll();
    previewed = false;
    try {
      in = new BufferedReader(new FileReader(filename));
    } catch (FileNotFoundException e) {
      Thread.currentThread().dumpStack();
      System.out.println(e.toString());
      return false;
    }
      
    keyframes = new Vector();
    while (current.read(in) != false) {
      /*System.out.println("FrameSet.loadPar read frame");*/
      if (method != -1 && method != current.method()) {
	System.out.println("You cannot mix CORNERS= and" + 
			   " CENTER-MAG= parameters");
	return false;
      }
      method = current.method();
      if (firstframe) {
	firstframe = false;
	if (current.steps != 0) {
	  System.out.println("Strange, the first frame has steps - zeroed");
	  current.steps = 0;
	}
      }
      addElement(current);
      /*System.out.println("Loaded:");
	current.write(System.out);*/
      current = new ParFrame();
    }
    return true;
  }

  void dump() {
    ParFrame frame;

    synchronized(keyframes) {
      Enumeration e = keyframes.elements();
      while (e.hasMoreElements()) {
	frame = (ParFrame)e.nextElement();
	frame.write(System.out);
      }
    }
  }
}








