/*-*-C-*-*/

import java.awt.*;
import java.awt.event.*;

public class IntField extends TextField implements FocusListener {
  int lastvalid = -1;
  int max, min;

  public IntField(int i) {
    this(i, Integer.MIN_VALUE, Integer.MAX_VALUE);
  }

  public IntField(int i, int min, int max) {
    super(2);
    setInt(i);
    this.min = min;
    this.max = max;
    addFocusListener(this);
    enableEvents(AWTEvent.KEY_EVENT_MASK);
  }

  public void setText(String s) {
    try {
      setInt(Integer.valueOf(s).intValue());
    } catch (NumberFormatException e) {
      super.setText("");
    }
  }

  void setInt(int s) {
    super.setText(Integer.toString(s));
    lastvalid = getInt();
  }

  public int getInt() {
    int s;
    try {
      s = Integer.valueOf(getText()).intValue();
    } catch (NumberFormatException e) {
      if (min < 0 && max > 0) {
	return 0;
      } else {
	return min;
      }
    }
    return s;
  }

  public void focusGained(FocusEvent e) {
    selectAll();
  }

  public void focusLost(FocusEvent e) {
    setInt(getInt());
  }

  protected void processKeyEvent(KeyEvent e) {
    /*System.out.println(e);*/
    if (!isEnabled()) {
      e.consume();
      return;
    }
    int key = e.getKeyCode();
    if (e.getID() == KeyEvent.KEY_PRESSED && isEnabled()) {
      if (key == KeyEvent.VK_ENTER) {
	setInt(getInt());
	e.consume();
      } else if (key == KeyEvent.VK_ADD) {
	setInt(getInt() + 1);
	e.consume();
      } else if (key == KeyEvent.VK_SUBTRACT) {
	setInt(getInt() - 1);
	e.consume();
      }
    } 
    super.processKeyEvent(e);
  }
}
