/*-*-C-*-*/

import java.awt.*;
import java.io.*;
import java.util.StringTokenizer;
import java.awt.image.MemoryImageSource;

public class Map extends Canvas {
  /*MapCanvas canvas = new MapCanvas();*/
  /*private Color values[];*/
  public int colors[];

  public Map() {
    super();
    doLayout();
  }

  public Map(String mapspec) {
    this();
    get(mapspec);
    doLayout();
  }

  public void setColor(int index, Color c) {
    setColor(index, c.getRed(), c.getGreen(), c.getBlue());
  }

  public void setColor(int index, int c) {
    if (colors == null) {
      colors = new int[256];
    }
    colors[index] = c;
  }

  public void setColor(int index, int r, int g, int b) {
    if (colors == null) {
      colors = new int[256];
    }
    colors[index] = makeColor(r, g, b);
  }

  public static int makeColor(int r, int g, int b) {
    return 0xFF << 24 | r << 16 | g << 8 | b;
  }

  public int getColor(int index) {
    return colors[index];
  }

  public int getRed(int index) {
    return parseRed(colors[index]);
  }

  public int getGreen(int index) {
    return parseGreen(colors[index]);
  }

  public static int parseRed(int c) {
    return c >> 16 & 0xFF;
  }

  public static int parseGreen(int c) {
    return c >> 8 & 0xFF;
  }

  public static int parseBlue(int c) {
    return c & 0xFF;
  }
  
  public int getBlue(int index) {
    return parseBlue(colors[index]);
  }

  public Dimension getPreferredSize() {
    return new Dimension(256, 20);
  }

  protected void draw(Graphics g, int offset, int row, int height) {
    int x, y;
    int source[];
    MemoryImageSource imgsrc;
    /*MediaTracker tracker = new MediaTracker(this);*/
    Image img;

    if (colors != null) {
      source = new int[256 * height];
      for (y=0; y < height; y++) {
	for (x=0; x < 256; x++) {
	  source[(y*256) + x] = colors[x];
	}
      }
      img = createImage(new MemoryImageSource(255, height, source, 0, 0));
      /*tracker.addImage(img, 0);

      try {
	tracker.waitForAll();
      } catch (InterruptedException ex) {
	System.out.println("ParFrame.setImage " + ex);
	setBackground(Color.red);
      }

      tracker.removeImage(img);*/
      /*g.drawImage(img, (getBounds().width - 256) /2 , 0, this);*/
      g.drawImage(img, offset, row, this);
    }
  }

  protected void draw(Graphics g, int row, int height) {
    draw(g, 0, row, height);
  }

  public void draw(Graphics g, int row) {
    draw(g, row, 1);
  }
    
  public void paint(Graphics g) {
    int x = Math.max(0, (getBounds().width - 255) / 2);

    draw(g, x, 0, 20);
  }

  public void update(Graphics g) {
    paint(g);
  }

  private boolean get(String mapspec) {
    int reps, i, offset = 0;
    int index = 0;
    StringBuffer repbuf;

    if (mapspec.charAt(0) == '@') {
      return load(mapspec.substring(1));
    } else {
      colors = new int[256];
      /*System.out.println("Decoding:"+mapspec);*/
      while ((offset < mapspec.length()) && 
	     (index < 256)) {
	/*System.out.println("Decoding color:"+index);*/
	setColor(index++, tripletDecode(mapspec, offset));
	offset += 3;
	if ((offset < mapspec.length()) && 
	    (index < 256) && 
	    (mapspec.charAt(offset) == '<')) {
	  offset++;
	  repbuf = new StringBuffer();
	  repbuf.append(mapspec.charAt(offset++));
	  while (mapspec.charAt(offset) != '>') {
	    repbuf.append(mapspec.charAt(offset++));
	  }

	  offset++;

	  reps = new Integer(repbuf.toString()).intValue();

	  /*System.out.println("Map.get index:"+index+" reps:" + reps);*/
	  setColor(index + reps - 1, tripletDecode(mapspec, offset));
	  for (i = index; i < index + reps; i++) {
	    /*System.out.println("Map.get decoding:\n"+ index + "\n" +
			       (index+reps) + "\n" + i);*/
	    setColor(i, avgColor(index -1, index + reps - 1, i));
	  }
	  index = index + reps;
	}		     
      }
      return true;
    }
  }

  public int avgColor(int from, int to, int here) {
    double scalar = (double)(here - from) / (to - from);
    
    return makeColor(Parameter.linear (getRed(from), getRed(to), scalar),
		     Parameter.linear (getGreen(from), getGreen(to), scalar),
		     Parameter.linear (getBlue(from), getBlue(to), scalar));
  }

  public int avgColor(int from, int to, double scalar) {
    return makeColor(Parameter.linear(parseRed(from), parseRed(to), scalar),
		     Parameter.linear(parseGreen(from),parseGreen(to), scalar),
		     Parameter.linear(parseBlue(from), parseBlue(to), scalar));
  }

  public void rotate(int howmuch) {
    if (howmuch != 0) {
      int newcolors[] = new int[256];
      /*Color newvalues[] = new Color[256];*/
      int i, from, to;
   
      newcolors[0] = colors[0];
      /*System.out.println("\nRotate:" + howmuch);*/
      for (i=0; i < 255; i++) {
	to = i+1;
	from = ((i + howmuch)%255) +1;
	newcolors[to] = colors[from];
	/*System.out.print("[" + from + "=" + to + "]  ");*/
      }
      colors = newcolors;
    }
  }

  public Map morph(Map next, double scalar) {
    int i;

    Map retval = new Map();
    retval.colors = new int[256];
    /*System.out.println("Map.morph scalar=" + scalar);*/
    for (i=0; i<256; i++) {
      retval.colors[i] = avgColor(colors[i], next.colors[i], scalar);
    }
    return retval;
  }


  public String toString() {
    int i;
    StringBuffer mapstring = new StringBuffer(768);

    for (i=0; i < 256; i++) {
      tripletEncode(mapstring, colors[i]);
    }
    return mapstring.toString();
  }

  public static void main(String argv[]) {
    for (int i=0; i<256; i++) {
      System.out.println("TEST:" + new Map()
			 .mapDecode(new Map()
				    .mapEncode(i)));
    }
  }

  public int tripletDecode(String mapspec, int offset) {
    return makeColor(mapDecode(mapspec.charAt(offset  )),
		     mapDecode(mapspec.charAt(offset+1)),
		     mapDecode(mapspec.charAt(offset+2)));
  }

  public void tripletEncode(StringBuffer buf, int c) {
    buf.append(mapEncode(parseRed(c)));
    buf.append(mapEncode(parseGreen(c)));
    buf.append(mapEncode(parseBlue(c)));
  }

  public int mapDecode(char c) {
    int retval;
    /*System.out.println("About to decod char:"+c);*/
    if (c >= 48 && c <= 57) { /* 0 - 9 */
      retval = (int)(c - 48);
    } else if (c >=65 && c <= 90) { /* A - Z */
      retval = (int)(c - 55);
    } else if (c >= 95 && c <= 122) { /* _ `  a - z */
      retval = (int)(c - 59);
    } else {
      System.out.println("ERROR: Map.mapdecode couldn't parse color");
      retval = -1;
    }
    return retval * 4;
  }

  public char mapEncode(int comp) {
    int i = comp / 4;
    char retval;
    /*System.out.print(i + "=");*/
    if (i >=0 && i <= 9) {
      retval = (char)(i + 48);
    } else if (i >= 10 && i <= 35) {
      retval = (char)(i + 55);
    } else if (i >= 36 && i <= 63) {
      retval = (char)(i + 59);
    } else {
      System.out.println("ERROR: Map.mapEncode couldn't parse color");
      retval = 48;
    }
    /*System.out.print(retval+": ");*/
    return retval;
  }

  public boolean load(String file) {
    int i, r, g, b;
    String rawline;
    doLayout();
    StringTokenizer line;
    BufferedReader in;

    System.out.println("Loading map from file:" + file);
    try {
      in = new BufferedReader(new FileReader(file));
    } catch (FileNotFoundException e) {
      Thread.currentThread().dumpStack();
      return false;
    }


    colors = new int[256];
    try {
      for (i=0; i<=255; i++) {
	rawline = in.readLine();
	line = new StringTokenizer(rawline, " \t");
	/*System.out.println(rawline);*/
	r = Integer.valueOf(line.nextToken()).intValue();
	g = Integer.valueOf(line.nextToken()).intValue();
	b = Integer.valueOf(line.nextToken()).intValue();
	colors[i] = makeColor(r, g, b);
	/*System.out.println(values[i]);*/
      }
    } catch (java.io.IOException e) {
      return false;
    }
    return true;
  }
}



