import java.awt.Color;

public class MapMaker {
  int bands = 20;
  int speed = 5;
  YigPoint points[];
  YigPoint sorted[];

  int r, g, b, or, og;

  public MapMaker() {
    reset();
  }

  public void reset() {
    int i;
    /*System.out.println("MapMaker.reset w/speed:" + speed);*/
    points = new YigPoint[bands];
    sorted = new YigPoint[bands];
    for (i=0; i < bands; i++) {
      sorted[i] = points[i] = new YigPoint(speed);
      points[i].randomize();
    }    
  }

  public void setSpeed(int s) {
    speed = s;
    reset();
  }

  public int getSpeed() {
    return speed;
  }

  public void setBands(int b) {
    bands = b;
    reset();
  }

  public int getBands() {
    return bands;
  }

  public void sort() {  /* bubble, that is */
    int i;
    boolean done = false;

    while (done == false) {
      done = true;
      for (i=1; i < bands; i++) {
	if (sorted[i-1].val(3) > sorted[i].val(3)) {
	  /*System.out.println("Swapping velocities");*/
	  sorted[i-1].swapBoth(sorted[i], 3);

	  /* Swapping r, g, b gives 'chunky' effect */
	  /*sorted[i-1].swapBoth(sorted[i], 0);
	  sorted[i-1].swapBoth(sorted[i], 1);
	  sorted[i-1].swapBoth(sorted[i], 2);*/
	  done = false;
	}

	/* Don't let two points sit on the same index */
	if (sorted[i-1].val(3) == sorted[i].val(3)) {
	  sorted[i].set(3, sorted[i].val(3) + 1);
	  done = false;
	}
      }
    }
    done = false;
    while (done == false) {
      done = true;
      for(i=1; i < bands; i++) {
	if (sorted[i].val(3) > YigPoint.MAXPOS) {
	  sorted[i].set(3, YigPoint.MAXPOS);
	  sorted[i].set(0, sorted[i-1].val(0));
	  sorted[i].set(1, sorted[i-1].val(1));
	  sorted[i].set(2, sorted[i-1].val(2));
	  done = false;
	}
      }
    }
  }

  public void fillMap(MetaMap map) {
    int i;
    for (i=0; i < map.length(); i++) {
      nextMap(map, i);
    }
  }

  public void nextMap(MetaMap map, int mapnum) {
    int i, from, to, step = 0;

    for (i=0; i < bands; i++) {
      points[i].move();
    }    
    sort();
    
    sorted[0].set(3, 1);
    sorted[bands-1].set(3, 255);

    /* Make the map 'rotation friendly' so the two ends meet up.
       The correct way to do this would be to let them wrap around,
       but it's all working like this already, and it seems fine so...
       oh well.
       */
    sorted[0].set(0, sorted[bands-1].val(0));
    sorted[0].set(1, sorted[bands-1].val(1));
    sorted[0].set(2, sorted[bands-1].val(2));

    map.setVal(mapnum, 0, 0);
    for (i=0; i < bands; i++) {
      map.setVal(mapnum, sorted[i].val(3),
		 MetaMap.makeColor(sorted[i].val(0),
				   sorted[i].val(1),
				   sorted[i].val(2)));
    }

    while (step < bands -1) {
      from = sorted[step++].val(3);
      to = sorted[step].val(3);
      map.makeRamp(mapnum, from, to);
    }
  }
}



