/*-*-C-*-*/
import java.io.*;
import java.awt.Canvas;
import java.awt.Image;
/*import java.awt.Event;  // for debugging output
import java.awt.Color;*/
import java.awt.image.MemoryImageSource;
import java.util.StringTokenizer;

public class MetaMap extends Canvas {
  private int values[];
  private int temp[] = new int[256];

  public MetaMap() { this(1); }

  public MetaMap(int s) {
    super();
    values = new int[s*256];
  }

  public int length() {
    return values.length / 256;
  }

  public void setVal(int mapnum, int colnum, int val) {
    values[(mapnum * 256) + colnum] = val;
  }

  public int getVal(int mapnum, int colnum) {
    return values[(mapnum * 256) + colnum];
  }

  public boolean load(String mapspec) { return load(0, mapspec); }

  public boolean load(int mapnum, String mapspec) {
    if (mapspec.charAt(0) == '@') {
      return readFile(mapnum, mapspec.substring(1));
    } else {
      return readString(mapnum, mapspec);
    }
  }

  public boolean readString(String map) { return readString(0, map); }

  public boolean readString(int mapnum, String mapspec) {
    int reps, offset = 0;
    int index = 0;
    StringBuffer repbuf;
    //System.out.println("Reading into " + mapnum + " string: " + mapspec);

    while ((offset < mapspec.length()) && (index < 256)) {
      /*System.out.println("setval1:" + mapnum + "/" + index + "=" + 
	new Color(tripletDecode(mapspec, offset)));*/
      setVal(mapnum, index++, tripletDecode(mapspec, offset));
      offset += 3;
      if ((offset < mapspec.length()) && 
	  (index < 256) && 
	  (mapspec.charAt(offset) == '<')) {
	offset++;
	repbuf = new StringBuffer();
	repbuf.append(mapspec.charAt(offset++));
	while (mapspec.charAt(offset) != '>') {
	  repbuf.append(mapspec.charAt(offset++));
	}

	offset++;
	reps = new Integer(repbuf.toString()).intValue();
	/*System.out.println("setval2:" +  mapnum + "/" + (index+reps) + "=" +
			   new Color(tripletDecode(mapspec, offset)));*/
	setVal(mapnum, index + reps, tripletDecode(mapspec, offset));
	makeRamp(mapnum, index -1, index + reps);
	index = index + reps;
      }		     
    }
    return true;
  }

  public boolean readFile(String file) { return readFile(0, file); }

  public boolean readFile(int mapnum, String file) {
    int i, r, g, b;
    String rawline;
    StringTokenizer line;
    BufferedReader in;

    try {
      in = new BufferedReader(new FileReader(file));
    } catch (FileNotFoundException e) {
      Thread.currentThread().dumpStack();
      return false;
    }

    try {
      for (i=0; i<=255; i++) {
	rawline = in.readLine();
	line = new StringTokenizer(rawline, " \t");
	r = Integer.valueOf(line.nextToken()).intValue();
	g = Integer.valueOf(line.nextToken()).intValue();
	b = Integer.valueOf(line.nextToken()).intValue();
	setVal(mapnum, i, makeColor(r, g, b));
      }
    } catch (java.io.IOException e) {
      return false;
    }
    return true;
  }

  public Image makeImage() {
    Image img = null;
    int y;

    img = createImage(new MemoryImageSource(256, length(), values, 0, 256));
    prepareImage(img, null);
    return img;
  }

  public Image makeImage(int mapnum, int height) {
    int source[] = new int[256 * height];
    Image img = null;
    int y;

    for (y=0; y < height; y++) {
      System.arraycopy(values, 0, source, y*256, 256); 
    }

    img = createImage(new MemoryImageSource(256, height, source, 0, 256));
    prepareImage(img, null);
    return img;
  }

  public static Image mapToImage(String mapspec, int height) {
    MetaMap map = new MetaMap();
    
    if (map.load(mapspec) == true) {
      return map.makeImage(0, height);
    } else {
      return null;
    }
  }

  /**
   * Generate a fractint-style mapspec from this map
   */
  public String toString() { return toString(0); }
    
  public String toString(int mapnum) {
    int i;
    StringBuffer mapstring = new StringBuffer(768);

    for (i=0; i < 256; i++) {
      tripletEncode(mapstring, getVal(mapnum, i));
    }
    return mapstring.toString();
  }

  /**
   * morph between two mapnums
   */
  public void morph(int from, int to) {
    int i, j;
    /*System.out.println("morphing from frame " + from +
      " to frame " + to);*/
    for (i = from + 1; i < to; i++) {
      for (j = 0; j < 256; j++) {
	/*System.out.println("frame " + i +
	  " color " + j +
	  " fromval=" + new Color(getVal(from, j)) +
	  " toval=" + new Color(getVal(to, j)));*/
	/*if (getVal(from, j) != getVal(to, j)) {
	  new Throwable().printStackTrace();
	  System.exit(1);
	  //throw new Exception("morphing between unlike colors");
	}*/
	setVal(i, j, avgColor(getVal(from, j), getVal(to, j),
			      makeScalar(from, to, i)));
      }
    }
  }

  /**
   * Make a ramp between two values
   */
  public void makeRamp(int from, int to) { makeRamp(0, from, to); }

  public void makeRamp(int mapnum, int from, int to) {
    int i;
    for (i=from + 1; i < to; i++) {
      setAvgColor(mapnum, i, from, to);
    }
  }

  /**
   * Rotate a single map plus or minus some number (don't move index 0)
   */
  public void rotate(int amount) {
    rotate(0, amount);
  }

  public void rotate(int mapnum, int amt) {
    int amount = amt % 256;
    if (amount == 0) return;

    System.arraycopy(values, mapnum * 256, temp, 0, 256);
    if (amount > 0) {
      System.arraycopy(temp, 1, 
		       values, mapnum * 256 + amount, 256 - amount);
      System.arraycopy(temp, 256 - amount,
		       values, mapnum * 256 + 1, amount - 1);
    } else {  /* we've already rulled out zero */
      amount = (amount * -1);
      System.arraycopy(temp, 1,
		       values, (mapnum * 256) + 256 - amount, amount);
      System.arraycopy(temp, amount + 1,
		       values, mapnum * 256 + 1, 256 - amount - 1);
    }
  }

  /**
   * Set the value of a color based on colors on either side
   */
  protected void setAvgColor(int mapnum, int here, int from, int to) {
    setVal(mapnum, here, avgColor(getVal(mapnum, from), getVal(mapnum, to),
				  makeScalar(from, to, here)));
  }

  /**
   * Figure out a scalar for the distance between here, from and to
   */
  protected static double makeScalar(int from, int to, int here) {
    return (double)(here - from) / (to - from);
  }

  /**
   * Average two packed colors based on a scalar
   */
  public static int avgColor(int from, int to, double scalar) {
    return makeColor(Parameter.linear (Red(from), Red(to), scalar),
		     Parameter.linear (Green(from), Green(to), scalar),
		     Parameter.linear (Blue(from), Blue(to), scalar));   
  }

  /** Functions to de-code the chanels of a packed color **/
  public static int Red(int c) { return c >> 16 & 0xFF; }
  public static int Green(int c) { return c >> 8 & 0xFF; }
  public static int Blue(int c) { return c & 0xFF; }

  /**
   * Create a packed color from chanels 
   */
  public static int makeColor(int r, int g, int b) {
    return 0xFF << 24 | r << 16 | g << 8 | b;
  }

  /**
   * Encode a color into fractint type char codes
   */
  public static void tripletEncode(StringBuffer buf, int c) {
    buf.append(mapEncode(Red(c)));
    buf.append(mapEncode(Green(c)));
    buf.append(mapEncode(Blue(c)));
  }

  /**
   * Encode a single chanel into it's fractint char equivilant
   */
  public static char mapEncode(int comp) {
    int i = comp / 4;
    if (i >=0 && i <= 9) {
      return (char)(i + 48);
    } else if (i >= 10 && i <= 35) {
      return (char)(i + 55);
    } else if (i >= 36 && i <= 63) {
      return (char)(i + 59);
    } else {
      System.out.println("ERROR: Map.mapEncode couldn't parse color");
      return 48;
    }
  }

  /**
   * Decode a color from a fractint mapspec
   */
  public int tripletDecode(String mapspec, int offset) {
    return makeColor(mapDecode(mapspec.charAt(offset  )),
		     mapDecode(mapspec.charAt(offset+1)),
		     mapDecode(mapspec.charAt(offset+2)));
  }

  /**
   * Decode one char into the correct int for a fractint mapspec
   */
  public static int mapDecode(char c) {
    if (c >= 48 && c <= 57) {              /* 0 - 9 */
      return (int)((c - 48) * 4);
    } else if (c >=65 && c <= 90) {        /* A - Z */
      return (int)((c - 55) * 4);
    } else if (c >= 95 && c <= 122) {      /* _ `  a - z */
      return (int)((c - 59) * 4);
    } else {
      System.out.println("ERROR: Map.mapdecode couldn't parse color");
      return -1;
    }
  }
}
