/*-*-C-*-*/

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.lang.*;

public class ParFrame extends Panel {
  static final int CORNERS = 0;
  static final int CENTER_MAG = 1;

  public boolean calculated = false;
  public int steps;
  Hashtable params = new Hashtable();
  String fullstring = new String();
  String comments = new String();
  static int gifnum = 0;
  ImageCanvas bitmap;
  Panel north = new Panel();
  Image mapimage;

  Dimension thumbsize = new Dimension(128 * 2, 96 * 2);

  public ParFrame() {
    this(false);
  }

  public ParFrame(boolean first) {
    super();
    
    setLayout(new BorderLayout());
    north.setLayout(new BorderLayout());
    add("North", north);
    if (FileManager.stdpar != null) {
      Enumeration e = FileManager.stdpar.elements();
      Parameter par;
      while (e.hasMoreElements()) {
	par = (Parameter)e.nextElement();
	params.put(par.name(), par);
      }
    }
  }

  public Enumeration elements() {
    return params.elements();
  }

  public String name() {
    return getval("name");
  }

  public ParFrame morph(ParFrame next, double scalar, String name) {
    ParFrame retval = new ParFrame();
    Parameter param, newparam;
    Enumeration e;
    
    newparam = new Parameter();
    newparam.read("name", name);
    retval.put(newparam);
    
    /* Parameters that are the same, or next is null */
    e = params.elements();
    while (e.hasMoreElements()) {
      param = ((Parameter)e.nextElement());
      if (!param.name().equals("colors")) {
	retval.put(param.morph(next.get(param.name()), scalar));
      }
    }
      
    /* Pick up leftovers where next has, but I don't */
    e = next.params.elements();
    while (e.hasMoreElements()) {
      param = ((Parameter)e.nextElement());
      if (retval.get(param.name()) == null) {
	newparam = param.morph(get(param.name()), scalar);
	if (newparam != null) {
	  retval.put(newparam);
	}
      }
    }
    return retval;
  }

  public void put(Parameter param) {
    if (param != null) {
      params.put(param.name(), param);
    }
  }
  
  public Parameter get(String name) {
    return (Parameter)params.get(name);
  }

  public boolean read(BufferedReader in) {
    String temp;
    int commentloc;
    
    try {
      do {
	temp = in.readLine();	
	if (temp == null) {
	  temp = "";
	}
	while (temp.indexOf('\\') != -1) {
	  temp = temp.substring(0, temp.indexOf('\\')).trim();
	  temp = temp + in.readLine().trim();
	} 
	
	commentloc = temp.indexOf(';');
	
	if (commentloc == -1) {
	  fullstring = fullstring + ' ' + temp;
	} else {
	  fullstring = fullstring + temp.substring(0, commentloc);
	  comments = comments + temp.substring(commentloc +1);
	}
      } while ((temp.indexOf('}') == -1) && (in.ready()));

    } catch (java.io.IOException e) {
      e.printStackTrace();
      System.out.println(e.toString());
      System.out.println("Failed to read ParFrame");
      return false;
    }

    if (temp.indexOf('}') == -1) {
      return false;
    } else {
      fullstring = fullstring.trim();
      comments = comments.trim();
      setsteps();
      readParams();
      if (params.get("colors") != null) {
	mapimage = MetaMap
	  .mapToImage(((Parameter)params.get("colors")).value(), 10);
	add("Center", new Sic(mapimage));
	doLayout();
	repaint();
      }

      return true;
    }
  }

  void setsteps() {
    int i = comments.indexOf(' ');
    String s;

    try {
      if (i == -1) {
	s = comments;
      } else {
	s = comments.substring(0, i);
      }
      steps = new Integer(s).intValue();
    } catch (Exception e) {
      steps = 0;
    }
  }

  void readParams() {
    StringTokenizer ps = new StringTokenizer(fullstring, " ");
    String temp;
    String name;
    Parameter param;
    Label templabel;

    param = new Parameter();
    param.read("name", ps.nextToken());
    params.put("name", param);

    if (steps != 0) {
      templabel = new Label("   " + name() + 
			    " (" + steps + " frames)", Label.CENTER);
    } else {
      templabel = new Label("   " + name(), Label.CENTER);
    }
    north.add("North", templabel);

    while (ps.hasMoreElements()) {
      temp = ps.nextToken().trim();
      try {
	if (temp.length() > 0) {
	  if (!temp.equals("{") && 
	      !temp.equals("reset") &&
	      !temp.equals("}")) {
	    param = new Parameter();
	    name = temp.substring(0, temp.indexOf('=')).toLowerCase();
	    if (name.equals("colors")) {
	      temp = temp.substring(temp.indexOf('=') + 1);
	    } else {
	      temp = temp.substring(temp.indexOf('=') + 1).toLowerCase();
	    }
	    if (!name.equals("reset") && !name.equals("viewwindows")) {
	      if (param.read(name, temp) == true) {
		params.put(name, param);
	      }
	    }
	  }
	}
      } catch (StringIndexOutOfBoundsException ex) {
	System.out.println(ex + 
	   "ParFrame.readparams:  Error parsing .par file.\n");
	write(System.out);
      }
    }
  }

  public void write(OutputStream o) {
    Parameter param;

    PrintStream out = new PrintStream(o);
    out.println(getval("name") + " {");
    Enumeration e = params.elements();
    while (e.hasMoreElements()) {
      param = (Parameter)e.nextElement();
      if (!param.name().equals("name")) {
	param.write(out);
      }
    }
    out.println("}\n");
  }

  public void setImage(String filename) {
    if (bitmap != null) {
      remove(bitmap);
    }
    north.add("South", bitmap = new ImageCanvas(filename, 2));
    doLayout();
  }

  public boolean preView() {
    String command;
    InputStream in;
    boolean retval;
    Parameter name;

    name = new Parameter();
    name.read("savename", FileManager.getNextOut());
    put(name);

    if (calc("128_preview")) {
      setImage(name.value());
      retval = true;
    } else {
      if (bitmap != null) {
	remove(bitmap);
      }
      retval = false;
    }
    new File(name.value()).delete();
    doLayout();
    return retval;
  }

  public boolean calc(String aspect) {
    if (get("savename") == null) {
      System.out.println("null savename on frame:");
      write(System.out);
      return false;
    } else {
      return calc(aspect, getval("savename"));
    }
  }

  public boolean calc(String aspect, String filename) {
    FileOutputStream out;
    File output;
    String command;
    Process p;

    output = new File(filename);

    String temp = filename.substring(0, filename.indexOf(".")) + ".par";
    int retval;
    
    if (output.exists()) {
      return true;
    }

    command = FileManager.lookup("<fractint_command>");
    if (command == null) {
      command = "fractint";
    }

    command = (command + " @aspect.par/" + aspect + " @" + temp + "/" + 
	       getval("name") + " savename=" + filename);
    /*System.out.println("About to run:" + command);*/
    try {
      out = new FileOutputStream(temp);
      write(out);
      out.close();
      p = Runtime.getRuntime().exec(command); 
      retval = p.waitFor();
      if (retval != 0) {
	System.out.println("fractint exited with code:" + retval);
      }
    } catch (java.io.IOException e) {
      System.out.println("Can't exec:" + command);
      retval = -1;
    } catch (java.lang.InterruptedException e) {
      e.printStackTrace();
      System.out.println(e.toString() + " in preView trying to wait");
      retval = -1;
    }
    new File(temp).delete();
    if (retval == 0) {
      return true;
    } else {
      return false;
    }    
  }

  public void stop() {
    System.out.println("I'd stop if I could.");
    /*
    if (p != null) {
      synchronized(p) {
	OutputStream out = p.getOutputStream();
	System.out.println("Killing process");
	try {
	  out.write(27);
	} catch (IOException e) {
	  System.out.println("Couldn't kill process");
	}
      }
    }
    */
  }

  public String getval(String k) {
    return getval(k, 1);
  }

  public String getval(String key, int which) {
    try {
      return ((String) ( ((Parameter)(params.get(key))) ).value(which) );
    } catch (java.lang.NullPointerException e) {
      return null;
    }
  }

  public int method() {
    if (getval("corners") != null) {
      return CORNERS;
    } else {
      return CENTER_MAG;
    }
  }

}

