import java.util.*;
import java.io.*;

public class Parameter {
  Vector values = null;

  public Parameter() {
    values = new Vector();
  }

  public String name() {
    return value(0);
  }

  public String value() {
    return value(1);
  }

  public String value(int i) {
    if (values != null && size() > i && values.elementAt(i) != null) {
      return (String)values.elementAt(i);
    } else {
      return null;
    }
  }

  public int size() {
    return values.size();
  }

  public Parameter morph(Parameter next, double scalar) {
    if (next == null) {
      return this;
    }
    if (name().equals("colors") ||
	name().equals("name")) {
      return null;
    } else if (name().equals("corners")) {
      /*System.out.println("Choosing corners.");*/
      return morphCorners(next, scalar);
    } else if (name().equals("center-mag")) {
      /*System.out.println("Choosing center-mag.");*/
      return morphCenter(next, scalar);
    } else if (name().equals("colors")) {
      /*System.out.println("Morphing colors");
      return morphColors(next, scalar);*/   /* do nothing */
      return null;
    } else {
      return morphLinear(next, scalar); 
    }
  }

  public void addElement(String val) {
    values.addElement(val);
  }

  public void write(PrintStream out) {
    /* Causes problems!  I don't like new things )-: */
    /*PrintStream out = new PrintStream(o);*/
    int i;
    StringBuffer output = new StringBuffer();

    output.append("   " + name() + "=" + value(1));
    for (i=2; (i< size()); i++) {
      output.append("/" + value(i));
    }
    output.append("\n");
    for (i=77; i<output.length(); i += 79) {
      output.insert(i, "\\\n   ");
    }
    out.print(output);
  }

  public boolean read(String name, String vals) {
    StringTokenizer vs;
    String value;

    /*values = new Vector();*/
    addElement(name);
    vs = new StringTokenizer(vals, "/");

    while (vs.hasMoreElements()) {
      value = vs.nextToken();
      addElement(value);
      /*System.out.println("Adding:" + value);*/
    }
    return true;
  }

  /* BEWARE, THERE BE MATH BELOW  - Morphing functions follow */

  Parameter morphLinear(Parameter next, double scalar) {
    int i;
    Parameter retval = new Parameter();
    double value, thisval, nextval;
    String strval;

    if ((this.size() != next.size()) || (!this.name().equals(next.name()))) {
      System.out.println("Parameter mis-match.\n " + 
			 name() + " size:"  + size() +
			 next.name() + " size:"  + next.size());
      return null;
    } else {
      retval.addElement(name());
      for (i=1; i < size(); i++) {
	/*System.out.println("this:" + value(i) + "\nnext:" + next.value(i));*/
	
	try {
	  thisval = new Double(value(i)).doubleValue();
	  nextval = new Double(next.value(i)).doubleValue();
	  
	  /*value = ( ((nextval - thisval)*scalar) + thisval);*/

	  if (FileManager.intparams.get(name()) == null) {
	    strval = linear(thisval, nextval, scalar);
	  } else {
	    strval = Integer.toString(linear((int)thisval, 
					     (int)nextval, scalar));
	  }
	  
	  /*System.out.println("start:" + value(i) +
	    " end:" + next.value(i) +
	    " scalar:" + scalar +
	    " here:" + strval);*/
	} catch (java.lang.NumberFormatException e) {
	  strval = value(i);
	}
	retval.addElement(strval);
      }
    }
    return retval;
  }

  /**
    *  exponential is a simplified version of dave's morphCorners function.
    *  it is used in the morphCenter function.
    *
    *  This is the tricky part, where we scale in an exponential curve from
    *  the point (0, 0) to the point (1, 1).  Depending on the zoom_factor
    *  (the ratios of the magnification), the curve will go slowly at first,
    *  then fast, or fast at first then slow.  These represent the two
    *  cases - xooming in or zooming out.  If the zoom_factor is one, there
    *  is no zoom, and we use a linear function, because all points are
    *  equal along the path.  
    *
    *  scalar is the input (on the x-axis) and the return value is the
    *  output (on the y-axis), thus the analogy to points (0, 0) and (1, 1).
    *  we always input a range of values from 0 to 1 and get out values from
    *  0 to 1, but if we put in 0, we should get out 0, and if we put in
    *  1, we should get out 1.
    *
    *       <<  INPUT vs. OUTPUT based on zoom_factor >>
    *
    *  zoom_factor > 1                   zoom_factor < 1
    *  |              *  *  *            |                   *
    *  |        *  *                     |                   * 
    *  |     *                           |                   *
    *  |   *                             |                  *
    *  |  *                              |                 *
    *  | *                               |               *
    *  | *                               |         *  *
    *  |*                                |*  *  *  
    *  +---------------------            +---------------------
    **/
  String exponential(double zoom_factor, double scalar, 
		     double me, double next) {

    double new_scalar;

    /* this is to avoid rounding errors that get amplified.  There must be
       a better way, but when the zoom factor is that close to 1, I don't
       think it makes any difference. */
    if (Math.abs(zoom_factor-1) < .0001) {
      /*System.out.println("Using linear");*/
      return linear(me, next, scalar);
    } else {
      new_scalar = (
		    (1 - Math.exp(scalar * Math.log(zoom_factor) ) ) 
		    / 
		    (1 - zoom_factor * scalar) );

      /*System.out.println("Parameter.exponential" + 
			 "\n  input:      "+scalar +
			 "\n  output:     "+new_scalar+
			 "\n  zoom factor:" + zoom_factor +
			 "\n  log:        " + Math.log(zoom_factor) +
			 "\n  exp(log):   " + Math.exp(scalar * 
						    Math.log(zoom_factor)) +
			 "\n  denominator:" + (1 - zoom_factor * scalar)
			 );*/

      if (Double.isNaN(new_scalar)) {
	/*System.out.println("Scalar is NaN");
	  System.out.println("Using linear");*/
	return linear(me, next, scalar);
      } else  if (new_scalar < 0 || new_scalar > 1) {
	/*System.out.println("New Scalar is out of range");
	System.out.println("Using linear");*/
	return linear(me, next, scalar);
      } else {
	return linear (me, next, new_scalar);
      }
    }
  }

  Parameter morphCorners(Parameter next, double scalar) {
    
    double thisxmin, thisxmax, thisymin, thisymax;
    double nextxmin, nextxmax, nextymin, nextymax;
    String xmin, xmax, ymin, ymax;
    double xscale, yscale;

    thisxmin = new Double(value(1)).doubleValue();
    thisxmax = new Double(value(2)).doubleValue();
    thisymin = new Double(value(3)).doubleValue();
    thisymax = new Double(value(4)).doubleValue();

    nextxmin = new Double(next.value(1)).doubleValue();
    nextxmax = new Double(next.value(2)).doubleValue();
    nextymin = new Double(next.value(3)).doubleValue();
    nextymax = new Double(next.value(4)).doubleValue();

    xscale = (nextxmax - nextxmin)/(thisxmax - thisxmin);
    yscale = (nextymax - nextymin)/(thisymax - thisymin);

    xmin = exponential(xscale, scalar, thisxmin, nextxmin);
    xmax = exponential(xscale, scalar, thisxmax, nextxmax);
    ymin = exponential(yscale, scalar, thisymin, nextymin);
    ymax = exponential(yscale, scalar, thisymax, nextymax);

    Parameter retval = new Parameter();
    retval.addElement("corners");
    retval.addElement(xmin);
    retval.addElement(xmax);
    retval.addElement(ymin);
    retval.addElement(ymax);
    return retval;
  }

  Parameter morphCenter(Parameter next, double scalar) {
    String Txmagfactor = value(4);
    String Trotation = value(5);
    String Tskew = value(6);

    String Nxmagfactor = next.value(4);
    String Nrotation = next.value(5);
    String Nskew = next.value(6);

    double Txctr = new Double(value(1)).doubleValue();
    double Tyctr = new Double(value(2)).doubleValue();
    double Nxctr = new Double(next.value(1)).doubleValue();
    double Nyctr = new Double(next.value(2)).doubleValue();
    double Tmag = new Double(value(3)).doubleValue();
    double Nmag = new Double(next.value(3)).doubleValue();

    double scale, cutoff, xcurr, ycurr;
    String X, Y, mag;

    if (Tmag < Nmag) {
      mag = exponential(Tmag / Nmag, 1-scalar, Nmag, Tmag);
      X = exponential(Tmag / Nmag, scalar, Txctr, Nxctr);
      Y = exponential(Tmag / Nmag, scalar, Tyctr, Nyctr);
    } else {
      mag = exponential(Nmag / Tmag, scalar, Tmag, Nmag);
      X = exponential(Nmag / Tmag, 1-scalar, Nxctr, Txctr);
      Y = exponential(Nmag / Tmag, 1-scalar, Nyctr, Tyctr);
    }

    Parameter retval = new Parameter();
    retval.addElement("center-mag");
    retval.addElement(X);
    retval.addElement(Y);
    retval.addElement(mag);

    /*System.out.println("Parameter.morphCenter morphing from:" +
		       Txctr +"x"+ Tyctr +" - "+ 
		       Nxctr +"x"+ Nyctr +"\n" +
		       "Tmag:" + Tmag + " Nmag:" + Nmag + "\n"
		       "NewX:" + X + " NewY: + Y);   */

    /* optional parameters */
    if (Txmagfactor != null || Nxmagfactor != null) {
      retval.addElement( linear("1", Txmagfactor, Nxmagfactor, scalar));
    }
    if (Trotation != null || Nrotation != null) {
      retval.addElement( linear("0", Trotation, Nrotation, scalar));
    }
    if (Tskew != null || Nskew != null) {
      retval.addElement( linear("0", Tskew, Nskew, scalar));
    }
    return retval;
  }
  
  String linear(String deflt, String me, String next, double scalar) {
    String retval;
    double medouble, nextdouble;

    if (scalar == 0) {
      if (me == null) {
	return deflt;
      } else {
	retval = me;
      }
    } else {
      if (me == null) {
	me = deflt;
      }
      if (next == null) {
	next = deflt;
      }
      medouble = new Double(me).doubleValue();
      nextdouble = new Double(next).doubleValue();

      retval = linear(medouble, nextdouble, scalar);

      if (retval.equals("NaN")) {
	retval = ("Linear creating NAN:\n" + 
		  "me:" + me + " next:" + next + " scalar" + scalar);
      }
    }
    return retval;
  }

  public static String linear(double me, double next, double scalar) {
    String strval;
    double val = (((next - me) * scalar) + me);
    // Detect integer values
    if (((int)val) == val) {
      strval = Integer.toString((int)val);
    } else {
      strval = Double.toString(val);
    }
    return strval;
  }

  public static int linear(int me, int next, double scalar) {
    int retval;
    retval = (int)( ( (double)(next - me) * scalar) + me);
    /*System.out.println("Parameter.linear (integer) me:"+ me +
      " next:"+next+" return:"+ retval +
      " scalar:"+scalar);*/
    return retval;
  }
}
