/*-*-C-*-*/

import java.awt.*;
import java.util.Enumeration;
import java.util.Vector;
import java.io.*;
import java.awt.image.*;
import java.awt.image.ImageObserver;
import java.net.*;

public class RenderMan extends ScrollPane {
  final static int PREVHEIGHT = 38;
  public Thread t;
  Panel display = new Panel();
  FrameSet keyframes;
  Filmer owner;
  String outfile = null;
  String aspect;
  int totalframes = 0, assigned = 0;
  public ParFrame frames[] = null;
  MetaMap allmaps = null;
  Renderer local;
  Dimension localsize = new Dimension(300, 150); 
  ServerSocket server = null;
  MapControls mapcontrols = null;
  Panel previews = new Panel();
  Sic mapimage = null;

  public RenderMan(MapControls m, Filmer o) {
    super();
    Panel p = new Panel();
    mapcontrols = m;
    owner = o;
    add("Center", display);
    display.setLayout(new BorderLayout());
    display.add("East", p);
    p.setLayout(new BorderLayout());
    p.add("North", previews);
    doLayout();
  }

  public void setFilm(FrameSet f, String a, String filename) {
    keyframes = f;
    aspect = a;
    outfile = filename;
  }

  public void newMap() {
    display.remove(mapimage);
    mapimage = null;
    allmaps = null;
  }

  public void reset() {
    assigned = 0;
  }

  public void doIt() {
    if (keyframes == null) {
      return;
    }

    t = Thread.currentThread();

    if (allmaps == null) {
      makeMap();
    }
    if (frames == null) {
      makeFrames();
    }

    if (t == Thread.currentThread()) {  /* check if aborted */
      if (outfile == null) {
	local = new Renderer(this, aspect);
	local.setBounds(100, 100, localsize.width, localsize.height);
	local.start();
	runServer();
      } else {
	writeFile(outfile);
      }
    }
  }

  public void preView() {
    int numpreviews, i;
    String file;
    t = Thread.currentThread();

    if (allmaps == null) {
      makeMap();
    }
    if (frames == null) {
      makeFrames();
    }
    numpreviews = (totalframes / PREVHEIGHT) +1;
    //if (numpreviews == 0) return;
    previews.removeAll();
    previews.setLayout(new GridLayout(numpreviews, 1));
    for (i=0; i < totalframes && t ==Thread.currentThread(); i += PREVHEIGHT) {
      if (frames[i].calc("50_tiny", file = FileManager.getNextOut())) {
	setMessage("making postage stamp " + 
		   (i/PREVHEIGHT + 1) + "/" + numpreviews);
	previews.add(new ImageCanvas(file));
	validate();
	mapimage.repaint();
      } else {
	System.out.println("Couldn't calc postage-stamp");
      }
    }
    t = null;
  }

  protected void countFrames() {
    Enumeration enum;

    if (keyframes != null) {
      totalframes = 0;
      enum = keyframes.elements();
      enum.nextElement();  // Pop the first one, cause it dosn't count.
      while (enum.hasMoreElements()) {
	totalframes += ((ParFrame)enum.nextElement()).steps;
      }
    }
  }

  protected void makeFrames() {
    int steps, i, filmframe=-1;
    ParFrame from, next, morphed;
    Parameter param;
    Enumeration enum;

    if (frames != null || keyframes == null) return;

    countFrames();
    if (allmaps == null) {
      makeMap();
    }
    
    frames = new ParFrame[totalframes];
    enum = keyframes.elements();

    from = (ParFrame)enum.nextElement();
    while (enum.hasMoreElements() && t == Thread.currentThread()) {
      System.out.println("Getting next keyframe");
      next = (ParFrame)enum.nextElement();
      steps = next.steps;
	
      for (i=0; i < steps && t == Thread.currentThread(); i++) {
	filmframe++;
	if (frames[filmframe] == null) {
	  setMessage("creating frames: " + (filmframe + 1) + 
		     "/" + totalframes);
	  System.out.println("creating frames: " + (filmframe + 1) + 
		     "/" + totalframes);
	  morphed = from.morph(next, (double)i /steps, 
			       Integer.toString(filmframe));
	    
	  /* Set savename */
	  param = new Parameter();
	  param.read("savename", "tmp" + 
		    FileManager.padNum(filmframe, 5) + ".gif");
	  morphed.put(param);

	  frames[filmframe] = morphed;
	}
      }
      from = next;
    }
    mergeMap();
    if (t != Thread.currentThread()) {
      frames = null;
    }
  }

  protected void makeMap() {
    if (keyframes == null) return;

    ParFrame here;
    Enumeration e = keyframes.elements();
    int filmframe = 0;
    int i;

    setMessage("making new map");
    countFrames();
    allmaps = new MetaMap(totalframes + 1);
    if (mapcontrols.isSmart()) {
      mapcontrols.getMapMaker().fillMap(allmaps);
    } else {
      do {
	here = (ParFrame)e.nextElement();
	filmframe += here.steps;
        allmaps.load(filmframe, here.getval("colors"));
	if (filmframe > 0) {
	  /*System.out.println("morphing:\n" +
			     allmaps.toString(filmframe - here.steps) + 
			     "\nwith\n" +
			     allmaps.toString(filmframe));*/
	  allmaps.morph(filmframe - here.steps, filmframe);
	}
      } while (e.hasMoreElements());
    }
    setMessage("rotating map");
    for (i = 0; i < allmaps.length(); i++) {
      allmaps.rotate(i, i*mapcontrols.getRotation());
    }
    if (mapimage != null) {
      display.remove(mapimage);
    }
    display.add("Center", mapimage = new Sic(allmaps.makeImage()));
    display.invalidate();
    validate();
    mapimage.repaint();
    mergeMap();
  }

  protected void mergeMap() {
    if (frames == null || allmaps == null) return;
    int i;
    Parameter param;

    setMessage("merging map with frames");
    for (i=0; i < totalframes; i++) {
      param = new Parameter();
      param.read("colors", allmaps.toString(i));
      if (frames[i] == null) {
	System.out.println("Null pointer on frame:" + i);
      } else {
	frames[i].put(param);
      }
    }
  }

  protected void writeFile(String filename) {
    int framenum;

    setMessage("writing file " + filename);
    try {
      FileOutputStream out = new FileOutputStream(filename);
      for (framenum = 0; framenum < frames.length; framenum++) {
	frames[framenum].write(out);
      }
      out.close();
      outfile = null;
      setMessage("done creating .par file");
    } catch (java.io.IOException e) {
      System.out.println("Renderman.doIt caught IO Exception\n" +
			 "trying to write output file" + filename);
    }
  }

  protected void runServer() {
    Socket client = null;
    DataOutputStream out;
    DataInputStream in;
    int request, framenum;
    int portnum;

    Thread.currentThread().setPriority(Thread.currentThread()
				       .getPriority() + 1);

    portnum = FileManager.intLookup("<server_port>");

    try {
      server = new ServerSocket(portnum);
      /*setMessage("server waiting for connections");*/
      while ( t == Thread.currentThread() ) {
	client = server.accept();
	out = new DataOutputStream(client.getOutputStream());
	in =  new DataInputStream(client.getInputStream());
	request = in.readInt();
	if (request == RenderClient.GET_NEXT_FRAME) {
	  framenum = getNextFrame();
	  if (framenum == -1) {
	    client.close();
	  } else {
	    out.writeBytes(aspect);
	    out.writeBytes("\n");
	    out.writeInt(framenum);
	    frames[framenum].write(out);
	    out.flush();
	  }
	} else if (request == RenderClient.DONE_WITH_FRAME) {
	  doneWithFrame(in.readInt());
	} 
	out.close();
	in.close();
	client.close();
      }
      server.close();
    } catch (IOException ex) {
    }
  }

  public synchronized void stop() {
    t = null;
    if (local != null) {
      local.stop();
      local = null;
    }
    if (server != null) {
      try {
	server.close();
      } catch (IOException ex) {
      }
    }
  }

  public synchronized int getNextFrame() {
    int i;

    /*owner.setMessage("Server assigned frame:" + assigned + 
      "/" + frames.length);*/
    if (assigned < frames.length) {
      return assigned++;
    } else {
      setMessage("cleaning up missed frames");
      for (i=0; i < frames.length; i++) {
	if (frames[i].calculated == false) {
	  return i;
	}
      }
      setMessage("all frames rendered");
      stop();
      return -1;
    }
  }

  public synchronized void doneWithFrame(int frame) {
    frames[frame].calculated = true;
    setMessage("finished frame:" + frame + "/" + (frames.length-1));
  }

  protected void setMessage(String s) {
    if (owner != null) {
      owner.setMessage(s);
    }
  }

  public Dimension getPreferredSize() {
    return new Dimension(256 + 50 + getVScrollbarWidth() + 4, 
			 Math.min(super.getPreferredSize().height,
				  Toolkit.getDefaultToolkit()
				  .getScreenSize().height));
  }

  /*public void finalize() {
    System.out.println("RenderMan finalized");
  }*/
}
