import java.lang.Math;
import java.util.Random;

public class YigPoint {
  final static int DIMENSIONS=4;
  final static int MAXPOS=255;
  final static double RNDFACTOR=.5;
  final static double DRIFTFACTOR = RNDFACTOR;
  final static double DRAG = 0; //RNDFACTOR / 20;
  final static Random rand = new Random((long)Math.random());
  int maxvel;

  int pos[] = new int[DIMENSIONS];
  double vel[] = new double[DIMENSIONS];

  public YigPoint() {
    this(5);
  }

  public YigPoint(int s) {
    maxvel = s;
  }

  public void randomize() {
    int i;
    for (i=0; i < DIMENSIONS; i++) {
      vel[i] = (int)(rand.nextDouble() * maxvel * 2 - maxvel);
      pos[i] = (int)(rand.nextDouble() * MAXPOS);
    }
    if (pos[0] > 128 && pos[1] > 128 && pos[2] > 128) {
      pos[(int)(rand.nextDouble() * 3)] = (int)(rand.nextDouble() * MAXPOS);
    }
    if (pos[0] < 128 && pos[1] < 128 && pos[2] < 128) {
      pos[(int)(rand.nextDouble() * 3)] = (int)(rand.nextDouble() * MAXPOS);
    }
  }

  public int val(int dim) {
    return pos[dim];
  }

  public void set(int dim, int val) {
    pos[dim] = val;
  }

  public void swapBoth(YigPoint other, int dim) {
    double vel = other.vel[dim];
    int pos = other.pos[dim];
    other.vel[dim] = this.vel[dim];
    other.pos[dim] = this.pos[dim];
    this.vel[dim] = vel;
    this.pos[dim] = pos;
  }

  public void move() {
    int i;
    for (i=0; i < DIMENSIONS; i++) {

      vel[i] += rand.nextDouble() * RNDFACTOR * 2 - RNDFACTOR;
      if (vel[i] > maxvel) {
	vel[i] = maxvel;
      } else if (vel[i] < (-1 * maxvel)) {
	vel[i] = (-1 * maxvel);
      }

      pos[i] += (int)vel[i];
      if (pos[i] > MAXPOS) {
	vel[i] = -1 * vel[i];
	pos[i] = (2 * MAXPOS) - pos[i];
      } else if (pos[i] < 0) {
	vel[i] = -1 * vel[i];
	pos[i] = -1 * pos[i];
      }
    }
    drift();
  }

  /* make the first three dimensions make 'neat' colors */
  public void drift() {
    int i, lowest = 0, highest = 0, lowval = 256, highval = -1;
      
    /* tend toward colors where two channels are high and one low 
       or where one is high and two are low.  Don't let all three
       sit in high or low territory. (this is verrrry neat!)
       */
    for (i = 0; i < 3; i++) {
      if (pos[i] > highval) {
	highval = pos[i];
	highest = i;
      }
      if (pos[i] < lowval) {
	lowval = pos[i];
	lowest = i;
      }
    }
    if (pos[highest]  < 128 && pos[lowest] < 128) { 
      vel[highest] += rand.nextDouble() * DRIFTFACTOR * 2 - DRIFTFACTOR;
    }
    if (pos[highest] > 128 && pos[lowest] < 128) {
      vel[lowest] -= rand.nextDouble() * DRIFTFACTOR * 2 - DRIFTFACTOR;
    }

    /* Give them all a little drag */
    for (i = 0; i < DIMENSIONS; i++) {
      vel[i] -= vel[i] * DRAG;
    }
  }

  public String toString() {
    int i;
    StringBuffer str = new StringBuffer();
    
    str.append("YigPoint values:");
    str.append(pos[0]);
    for (i = 1; i < DIMENSIONS; i++) {
      str.append("/");
      str.append(pos[i]);
    }
    return str.toString();
  }
}
